@@ -24,8 +24,8 @@
  *   are deemed to be part of the source code.
  *
  * References:
- *  - IBM "Using the BIOS Build ID to identify IBM Thinkpad systems"
- *    Revision 2005-03-02
+ *  - IBM "Using the BIOS Build ID to identify Thinkpad systems"
+ *    Revision 2006-01-31
  *    http://www-307.ibm.com/pc/support/site.wss/MIGR-45120.html
  *
  * Notes:
@@ -37,24 +37,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <getopt.h>
 
 #include "version.h"
 #include "config.h"
 #include "types.h"
 #include "util.h"
-
-/* Options are global */
-struct opt
-{
-	const char* devmem;
-	unsigned int flags;
-};
-static struct opt opt;
-
-#define FLAG_VERSION            (1<<0)
-#define FLAG_HELP               (1<<1)
-#define FLAG_DUMP               (1<<2)
+#include "vpdopt.h"
 
 static const char *product_name(const char *id)
 {
@@ -123,6 +111,8 @@ static const char *product_name(const char *id)
 		"KZ", "Thinkpad T21", /* fixed 2003-11-29 (IBM) */
 		"M1", "eServer xSeries 325", /* added 2005-04-26,
 		                                reported by Myke Olson */
+		"NR", "eServer xSeries 236", /* added 2005-10-26,
+		                                reported by Klaus Muth */
  		"NT", "Netfinity 3000", /* added 2005-05-25,
  		                           reported by Bernd Krumboeck */
 		"NV", "PC 300PL", /* added 2004-04-02,
@@ -150,6 +140,8 @@ static const char *product_name(const char *id)
 		                                reported by Doug Brenner */
 		"TT", "eServer xSeries 330", /* added 2003-12-03,
 		                                reported by Hugues Lepesant */
+		"VI", "eServer xSeries 440", /* added 2005-08-25,
+		                                reported by Torsten Seemann */
 		"ZR", "eServer xSeries 200", /* added 2005-05-25,
 		                                reported by Bernd Krumboeck */
 		"10", "Thinkpad A21e or A22e", /* Celeron models */
@@ -180,10 +172,15 @@ static const char *product_name(const char *id)
 		                                                     updated 2005-03-08 (IBM) */
 		"1S", "Thinkpad R40e", /* added 2003-11-29 (IBM) */
 		"1T", "Thinkpad G40",
+		"1U", "Thinkpad X40", /* added 2005-06-24 (IBM) */
 		"1V", "Thinkpad R51", /* added 2005-03-08 (IBM),
 		                         confirmed by Ingo van Lil */
 		"1W", "Thinkpad R50e", /* added 2005-02-17 (IBM) */
-		"1Y", "Thinkpad T43/p", /* added 2005-02-17 (IBM) */
+		"1X", "Thinkpad G41", /* added 2005-06-24 (IBM),
+		                         types 2881, 2882 and 2886 */
+		"1Y", "Thinkpad T43/p", /* added 2005-02-17 (IBM),
+		                           updated 2005-06-24 (IBM),
+		                           types 2668, 2669, 2678, 2679, 2686 and 2687 */
 		"20", "Netvista A22p or M41", /* added 2003-10-09,
 		                                 updated 2004-02-23,
 		                                 updated 2004-03-24 */
@@ -193,6 +190,18 @@ static const char *product_name(const char *id)
 		                            reported by Rafael Avila de Espindola */
 		"2C", "Thinkcentre A50", /* added 2005-02-24,
 		                            reported by Tomek Mateja */
+		"70", "Thinkpad T43/p or R52", /* added 2005-06-24,
+		                                  T43/p types 1871, 1872, 1873, 1874,1875 and 1876,
+		                                  R52 types 1858, 1859, 1860, 1861, 1862 and 1863 */
+		"74", "Thinkpad X41", /* added 2005-06-24 (IBM) */
+		"75", "Thinkpad X41 Tablet", /* added 2005-06-24 (IBM) */
+		"76", "Thinkpad R52", /* added 2005-06-24,
+		                         types 1846, 1847, 1848, 1849, 1850 and 1870 */
+		"77", "Thinkpad Z60m/t", /* added 2005-10-06 */
+		"78", "Thinkpad R51e", /* added 2005-10-06 */
+		"7B", "Thinkpad X60/s", /* added 2006-01-31,
+		                           X60 types 1706, 1707, 1708, 1709, 2509 and 2510,
+		                           X60s types 1702, 1703, 1704, 1705, 2507, 2508 and 2533 */
 		NULL, "Unknown, please report!"
 	};
 	
@@ -212,7 +221,8 @@ static void print_entry(const char *name, const u8 *p, size_t len)
 {
 	size_t i;
 	
-	printf("%s: ", name);
+	if(name!=NULL)
+		printf("%s: ", name);
 	for(i=0; i<len; i++)
 	{
 		/* ASCII filtering */
@@ -253,8 +263,8 @@ static int decode(const u8 *p)
 	if(p[5]<0x30)
 		return 0;
 	
-	/* XSeries have longer records. */
-	if(!(p[5]>=0x46 && checksum(p, 0x46))
+	/* XSeries have longer records, exact length seems to vary. */
+	if(!(p[5]>=0x45 && checksum(p, p[5]))
 	/* Some Netvista seem to work with this. */
 	&& !(checksum(p, 0x30))
 	/* The Thinkpad/Thinkcentre checksum does *not* include the first
@@ -264,73 +274,39 @@ static int decode(const u8 *p)
 		/* A few systems have a bad checksum (xSeries 325, 330, 335
 		   and 345 with early BIOS) but the record is otherwise
 		   valid. */
-		printf("Bad checksum! Please report.\n");
+		if(!(opt.flags & FLAG_QUIET))
+			printf("Bad checksum! Please report.\n");
 	}
 	
+	if(opt.string!=NULL)
+	{
+		if(opt.string->offset+opt.string->len<p[5])
+			print_entry(NULL, p+opt.string->offset,
+			            opt.string->len);
+		return 1;
+	}
+
 	print_entry("BIOS Build ID", p+0x0D, 9);
 	printf("Product Name: %s\n", product_name((const char *)(p+0x0D)));
 	print_entry("Box Serial Number", p+0x16, 7);
 	print_entry("Motherboard Serial Number", p+0x1D, 11);
 	print_entry("Machine Type/Model", p+0x28, 7);
 	
-	if(p[5]<0x45)
+	if(p[5]<0x44)
 		return 1;
 	
 	print_entry("BIOS Release Date", p+0x30, 8);
-	print_entry("Default Flash Image File Name", p+0x38, 13);
+	print_entry("Default Flash Image File Name", p+0x38, 12);
+
+	if(p[5]>=0x46 && p[0x44]!=0x00)
+	{
+		printf("%s: %u (Please report!)\n", "BIOS Revision",
+		       p[0x44]);
+	}
 	
 	return 1;
 }
 
-/* Return -1 on error, 0 on success */
-static int parse_command_line(int argc, char * const argv[])
-{
-	int option;
-	const char *optstring = "d:huV";
-	struct option longopts[]={
-		{ "dev-mem", required_argument, NULL, 'd' },
-		{ "help", no_argument, NULL, 'h' },
-		{ "dump", no_argument, NULL, 'u' },
-		{ "version", no_argument, NULL, 'V' },
-		{ 0, 0, 0, 0 }
-	};
-
-	while((option=getopt_long(argc, argv, optstring, longopts, NULL))!=-1)
-		switch(option)
-		{
-			case 'd':
-				opt.devmem=optarg;
-				break;
-			case 'h':
-				opt.flags|=FLAG_HELP;
-				break;
-			case 'u':
-				opt.flags|=FLAG_DUMP;
-				break;
-			case 'V':
-				opt.flags|=FLAG_VERSION;
-				break;
-			case ':':
-			case '?':
-				return -1;
-		}
-
-	return 0;
-}
-
-static void print_help(void)
-{
-	static const char *help=
-		"Usage: vpddecode [OPTIONS]\n"
-		"Options are:\n"
-		" -d, --dev-mem FILE     Read memory from device FILE (default: " DEFAULT_MEM_DEV ")\n"
-		" -h, --help             Display this help text and exit\n"
-		" -u, --dump             Do not decode the VPD records\n"
-		" -V, --version          Display the version and exit\n";
-	
-	printf("%s", help);
-}
-
 int main(int argc, char * const argv[])
 {
 	u8 *buf;
@@ -362,18 +338,22 @@ int main(int argc, char * const argv[])
 		return 0;
 	}
 
-	printf("# vpddecode %s\n", VERSION);
+	if(!(opt.flags & FLAG_QUIET))
+		printf("# vpddecode %s\n", VERSION);
 
 	if((buf=mem_chunk(0xF0000, 0x10000, opt.devmem))==NULL)
 		exit(1);
 
-	for(fp=0; fp<=0xFFF0; fp+=16)
+	for(fp=0; fp<=0xFFF0; fp+=4)
 	{
 		u8 *p=buf+fp;
 
 		if(memcmp((char *)p, "\252\125VPD", 5)==0
 		&& fp+p[5]-1<=0xFFFF)
 		{
+			if(fp%16 && !(opt.flags & FLAG_QUIET))
+				printf("Unaligned address (%#lx), please report!\n",
+				       0xf0000+fp);
 			if(opt.flags & FLAG_DUMP)
 			{
 				dump(p, p[5]);
@@ -389,7 +369,7 @@ int main(int argc, char * const argv[])
 
 	free(buf);
 	
-	if(!found)
+	if(!found && !(opt.flags && FLAG_QUIET))
 		printf("# No VPD structure found, sorry.\n");
 	
 	return 0;
